/*=====================================================================
  This scripts is inteaded to be used by CSS to analyze latency issues for Always On.

  Use the CTRL-SHIFT-M macro substitution

  Parameters:
  ----------
  Analysis Database -- Database used to store xevents tables.
  User Database -- Primary replica database.

  Notes:
  -----
  The script is expected to run on the same server that hosts the AG primary database.

  It is expected to import_template.sql to build DMReplicaEvents table.

  Output:
  ------
  This script analyzes different stages of log block. It helps you to narrow down where you need to focus.
  It returns the following. The time is measured in micro-sec.:
  		-- Average time for commit, this includes local flus + group commit + remote harden
		-- Average remote harden time
		-- Average time spent for group commit
		-- Average time spent on compression
		-- Average time spent inside the primary to prepare the message on put on the wire

======================================================================*/
SET NOCOUNT ON
USE [tempdb] 
GO

DECLARE @Db_Id AS INT,
		@Dbr_Id AS uniqueidentifier,
		@AvgCommitTime AS BIGINT,
		@AvgRemoteHardenTime AS BIGINT,
		@AvgGroupCommitTime AS BIGINT,
		@AvgCompressionTime AS BIGINT,
		@AvgSendTime AS BIGINT,
		@AvgReceiveTime AS BIGINT


SELECT @Db_Id = d.database_id, @Dbr_Id = drs.group_database_id
FROM sys.databases d 
	inner join sys.dm_hadr_database_replica_states drs 
	on drs.database_id = d.database_id
WHERE name = N'TestAG'

SELECT @AvgCommitTime = AVG(processing_time)
FROM DMReplicaEvents dme
WHERE event_name = 'recovery_unit_harden_log_timestamps' AND database_id = @Db_Id

SELECT @AvgRemoteHardenTime = AVG(processing_time)
FROM DMReplicaEvents dme
WHERE event_name = 'hadr_db_commit_mgr_harden' AND database_id = @Db_Id

SELECT @AvgGroupCommitTime = AVG(processing_time)
FROM DMReplicaEvents dme
WHERE event_name = 'hadr_log_block_group_commit' AND database_id = @Db_Id

SELECT @AvgCompressionTime = AVG(processing_time)
FROM DMReplicaEvents dme
WHERE event_name = 'hadr_log_block_compression' AND database_id = @Db_Id

-- Note: This might not be very accurate because it is not filtered for a specific replica or database
-- We are working on fixing this.
SELECT @AvgSendTime = AVG(processing_time)
FROM DMReplicaEvents dme
WHERE event_name = 'hadr_log_block_send_complete'

SELECT
		-- Average time for commit, this includes local plus + group commit + remote harden
		@AvgCommitTime AS [Avg. Commit Time],
		-- Average remote harden time
		@AvgRemoteHardenTime AS [Avg. Remote Harden],
		-- Average time spent for group commit
		@AvgGroupCommitTime AS [Avg. Group Commit Time],
		-- Average time spent on compression
		@AvgCompressionTime AS [Avg. Compression Time],
		-- Average time spent inside the primary to prepare the message on put on the wire
		@AvgSendTime AS [Avg. Send Time]