/*=====================================================================
  This scripts is inteaded to be used by CSS to analyze latency issues for Always On.

  Use the CTRL-SHIFT-M macro substitution

  Parameters:
  ----------
  Analysis Database -- Database used to store xevents tables.
  User Database -- Secondary replica database.

  Notes:
  -----
  The script is expected to run on the same server that hosts the AG secondary database.

  It is expected to import_template.sql to build DMReplicaEvents table.

  Output:
  ------
  This script analyzes different stages of log block. It helps you to narrow down where you need to focus.
  It returns the following. The time is measured in micro-sec.:
  		-- Average time for apply the message to disk
		-- Average time spent on decompression
		-- Average time spent inside the primary to prepare the lsn message on put on the wire
		-- Average time spent to receive time and route the message

======================================================================*/
SET NOCOUNT ON
USE [tempdb]
GO

DECLARE @Db_Id AS INT,
		@Dbr_Id AS uniqueidentifier,
		@AvgApplyTime AS BIGINT,
		@AvgDecompressionTime AS BIGINT,
		@AvgSendTime AS BIGINT,
		@AvgReceiveTime AS BIGINT

SELECT @Db_Id = d.database_id, @Dbr_Id = drs.group_database_id
FROM sys.databases d 
	inner join sys.dm_hadr_database_replica_states drs 
	on drs.database_id = d.database_id
WHERE name = N'TestAG'

SELECT @AvgApplyTime = AVG(processing_time)
FROM DMReplicaEvents dme
WHERE event_name = 'hadr_apply_log_block' AND database_replica_id = @Dbr_Id

SELECT @AvgDecompressionTime = AVG(processing_time)
FROM DMReplicaEvents dme
WHERE event_name = 'hadr_log_block_decompression' AND database_id = @Db_Id

SELECT @AvgReceiveTime = AVG(a.start_timestamp - r.start_timestamp)
FROM
(SELECT start_timestamp, log_block_id
FROM DMReplicaEvents
WHERE event_name = 'hadr_transport_receive_log_block_message' AND database_replica_id = @Dbr_Id and mode = 1) r
inner join 
(SELECT start_timestamp, log_block_id
FROM DMReplicaEvents
WHERE event_name = 'hadr_apply_log_block' AND database_replica_id = @Dbr_Id and mode = 1) a
on r.log_block_id = a.log_block_id 

-- Note: This might not be very accurate because it is not filtered for a specific replica or database
-- We are working on fixing this.
SELECT @AvgSendTime = AVG(processing_time)
FROM DMReplicaEvents dme
WHERE event_name = 'hadr_lsn_send_complete'

SELECT
		-- Average receive time
		@AvgReceiveTime as [Avg. Receive Time],
		-- Average time for apply the message to disk
		@AvgApplyTime AS [Avg. Apply Time],
		-- Average time spent on decompression
		@AvgDecompressionTime AS [Avg. Decompression Time],
		-- Average time spent inside the primary to prepare the lsn message on put on the wire
		@AvgSendTime AS [Avg. Send Time]

